/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
USART RX Input code for LiveLight V3 Firmware 						 _____  __   __
Written by Henri Rantanen											|  __ \ \ \ / /
																	| |__) | \ V /
Date		28.4.2014												|  _  /   > <  
Version		3.0.1													| | \ \  / . \ 
Target		PCB Version 5, Atmel ATMEGA32-16U						|_|  \_\/_/ \_\

************************************************************************************/
#include "settings.h"

/* Data input protocol example
	+--------+-------+---------+--------+-------+---------+--------+-------+----------/
	| Header | Red 1 | Green 1 | Blue 1 | Red 2 | Green 2 | Blue 2 | Red 3 | Green3  /
	+--------+-------+---------+--------+-------+---------+--------+-------+--------/
*/

ISR(USART_RXC_vect)							// UART Receive interrupt
{	
	// Reset shutdown timer
	ResetShutdownTimer();
	
	// Read header byte from EEPROM
	const uint8_t HeaderByte = EE_PREFIX; 
	
	// Read black level from EEPROM
	const uint8_t BlackLevel = EE_BLACK_LEVEL;
	
	// Read USART to receiver buffer
	volatile uint8_t buffer = UDR;
	
	// Buffer Index
	static volatile uint8_t i = 0;	
	
							
	// Check if input is valid or reset index
	if (i > CHANNELS) i = 0;
	
	if (RXBuffer[0] != HeaderByte)
	{
		if (RXBuffer[0] != (HeaderByte + 1)) i = 0;
	}
		
	// Read header byte
	if (i == 0) RXBuffer[0] = buffer;
	
	// Read color data
	else 										
	{
		// Apply minimum brightness
		if (buffer <= BlackLevel) RXBuffer[i] = BlackLevel;
		
		// Smooth input by calculating average from current and new color
		else RXBuffer[i] = Average(RXBuffer[i], buffer);	
	}

	// Grow index if header is OK
	if (RXBuffer[0] == HeaderByte || RXBuffer[0] == HeaderByte + 1) i++;	
	else i = 0;
}


 
uint8_t Average(uint8_t old, uint8_t new)
{		
	// Aggressive attack if change is great enough
	// if (new == 0x00) return new;	// Skip to black
	// if (new == 0xFF) return new;	// Skip to white
		
	// Calculate dynamic moving average (A + B) / 2 = C via bit shifting (FAST)
	if (new < 0xFF)	new = ((old << 7) + (new << 7)) >> 8;
	if (new < 0x80)	new = ((old << 7) + (new << 7)) >> 8;
	if (new < 0x40)	new = ((old << 7) + (new << 7)) >> 8;
	if (new < 0x20)	new = ((old << 7) + (new << 7)) >> 8;
			 				
	return new;
}


void Flush_USART( void )
{
	// Read everything in USART buffer to garbage until its empty
	uint8_t dummy;
	while ( UCSRA & (1<<RXC) ) dummy = UDR;
}